// DCD CSS monitor, programmed by mdx... All rights reserved

// include files
#include <windows.h>
#include <stdio.h>
#include <commctrl.h>
#include "aspi\wnaspi32.h"
#include "aspi\scsidefs.h"
#include "resource.h"

#define	OrgDLLName	"WNASPI32.DLL"

// public vars
HINSTANCE		OrgDLL = NULL;										// Handle to original (WNASPI32) DLL
DWORD			OrgVer = 1;											// Original (or fake) DLL version
char			msg[255];											// Dummy buffer for messages etc.
char			OrgPath[MAX_PATH];
int				Counter;
FILE			*hOutLog;
HINSTANCE		hInst;												// Handle to this DLL instance
volatile HANDLE	hThread = NULL;										// Handle to extra thread
volatile BOOL	StillRunning = TRUE;								// Flag if status Window is still open
volatile HWND	ListBox = 0, CloseBtn;								// Window handels for two most important controls

// DLL exported function declarations
_declspec(dllexport)	DWORD		GetASPI32DLLVersion(void);
_declspec(dllexport)	DWORD		GetASPI32SupportInfo(void);
_declspec(dllexport)	DWORD		SendASPI32Command(LPSRB);

// Original DLL imported functions
DWORD	(*lpGetSupportInfo)(void);
DWORD	(*lpSendCommand)(LPSRB);

// normal function declarations
BOOL WINAPI			DllMain(HINSTANCE, DWORD, LPVOID);
DWORD WINAPI		DialogThread(LPVOID);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
void				AddText(LPSTR);
void				AddTimedText(LPSTR);
BOOL				LoadOrgDLL(void);
BOOL				InitDLL(void);
char				*GetErrDesc(DWORD);
void				CmdDesc(LPSRB);
void				AfterCmd(LPSRB);
void				OutputToFile(char *, char *, int, int);


// DLL entry point
BOOL WINAPI	DllMain(HINSTANCE hInstDll, DWORD fdwReason, LPVOID reserved)
{
	switch(fdwReason)
	{
		case DLL_PROCESS_ATTACH:													// DLL file is opened
			Counter = 0;
			hInst = hInstDll;

			hOutLog = fopen("DVDCSSTrace.log", "wt");
			
			GetSystemDirectory(OrgPath, MAX_PATH);
			strcat(OrgPath, "\\");
			strcat(OrgPath, OrgDLLName);

			if (!LoadOrgDLL())
				return FALSE;
			break;

		case DLL_PROCESS_DETACH:													// DLL file is closed, free up
			fclose(hOutLog);

			EnableWindow(CloseBtn, TRUE);

			if (OrgDLL != NULL)
				FreeLibrary(OrgDLL);

			while(StillRunning)
				ResumeThread(hThread);
			break;
	}

	return TRUE;
}

// Dialog thread, this thread creates the monitor dialog
DWORD WINAPI	DialogThread(HINSTANCE	dllInst)
{
	DialogBox(dllInst, MAKEINTRESOURCE(IDD_DIALOG), NULL, (DLGPROC) WndProc);

	StillRunning = FALSE;
	return 0;
}

// Window message handler
LRESULT CALLBACK	WndProc(HWND hWnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	switch(iMsg)
	{
		case WM_INITDIALOG:
			ListBox = GetDlgItem(hWnd, IDC_LIST);
			CloseBtn = GetDlgItem(hWnd, IDCLOSEBTN);
			break;

		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case IDCLOSEBTN:
					EndDialog(hWnd, TRUE);
					break;
			}
			break;

		default:
			return FALSE;
	}

	return TRUE;
}

// Get number of installed busses, through old ASPI DLL
DWORD	GetASPI32SupportInfo(void)
{
	DWORD	returnVal;
	
	if (hThread == NULL)
		if (!InitDLL())
			return (SS_INSUFFICIENT_RESOURCES << 8);		// Convert to HIGHBYTE

	AddText("");
	AddText("GetASPI32SupportInfo");
	returnVal = lpGetSupportInfo();

	wsprintf(msg, "returned:\t%s", GetErrDesc((DWORD)HIBYTE(LOWORD(returnVal))));
	AddText(msg);
	wsprintf(msg, "\tNr of SCSI busses found: %u", LOBYTE(LOWORD(returnVal)));
	AddText(msg);

	return returnVal;
}

// Send command to ASPI host, through old ASPI DLL
DWORD	SendASPI32Command(LPSRB srb)
{
	DWORD			returnVal;
	SRB_ExecSCSICmd	*scsicmd;
	HANDLE			hOldEvent, hNewEvent = NULL;

	CmdDesc(srb);

	// Setup event handling if needed
	scsicmd = (LPSRB_ExecSCSICmd) srb;
	if (scsicmd->SRB_Flags && 0x40 == 0x40)
	{
		hNewEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
		if (hNewEvent)
		{
			hOldEvent = (HANDLE) scsicmd->SRB_PostProc;
			scsicmd->SRB_PostProc = (LPVOID) hNewEvent;
		}
	}
	
	// Execute the SCSI command
	returnVal = lpSendCommand(srb);
	
	// Wait for completion event if needed
	if (hNewEvent != NULL)
	{
		WaitForSingleObject(hNewEvent, INFINITE);
		CloseHandle(hNewEvent);
	}
	
	// If errors occured, exit DLL
	if ((returnVal != SS_COMP) && (returnVal != SS_PENDING))
	{
		AddText("ERRORS occured!");

		if (hNewEvent != NULL)
			SetEvent(hOldEvent);

		return returnVal;
	}

	// Display data that was returned if needed
	AfterCmd(srb);
	
	// Notify program that called DLL if needed
	if (hNewEvent != NULL)
	{
		CloseHandle(hNewEvent);
		SetEvent(hOldEvent);
	}

	return returnVal;
}

// Return the original (or this) DLL version
DWORD	GetASPI32DLLVersion(void)
{
	wsprintf(msg, "          returned: %d (0x%X)", OrgVer, OrgVer);
	AddText("");
	AddText("GetASPI32DLLVersion");
	AddText(msg);

	return OrgVer;
}

char	*GetErrDesc(DWORD err)
{
	switch(err)
	{
		case SS_PENDING:				return "SS_PENDING, SRB being processed";
		case SS_COMP:					return "SS_COMP, completed without error";
		case SS_ABORTED:				return "SS_ABORTED, aborted";
		case SS_ABORT_FAIL:				return "SS_ABORT_FAIL, unable to abort";
		case SS_ERR:					return "SS_ERR, completed with error";
		case SS_INVALID_CMD:			return "SS_INVALID_CMD, invalid ASPI command";
		case SS_INVALID_HA:				return "SS_INVALID_HA, invalid host adapter number";
		case SS_NO_DEVICE:				return "SS_NO_DEVICE, SCSI device not installed";
		case SS_INVALID_SRB:			return "SS_INVALID_SRB, invalid parameter set in SRB";
		case SS_BUFFER_ALIGN:			return "SS_BUFFER_ALIGN, buffer not correctly aligned";
		case SS_ILLEGAL_MODE:			return "SS_ILLEGAL_MODE, unsupported Windows mode";
		case SS_NO_ASPI:				return "SS_NO_ASPI, no ASPI manager installed";
		case SS_FAILED_INIT:			return "SS_FAILED_INIT, ASPI failed to init";
		case SS_ASPI_IS_BUSY:			return "SS_ASPI_IS_BUSY, no resources available to exec cmd";
		case SS_BUFFER_TO_BIG:			return "SS_BUFFER_TO_BIG, buffer size to big to handle";
		case SS_MISMATCHED_COMPONENTS:	return "SS_MISMATCHED_COMPONENTS, the ASPI files don't version check";
		case SS_NO_ADAPTERS:			return "SS_NO_ADAPTERS, no host adapters to manage";
		case SS_INSUFFICIENT_RESOURCES:	return "SS_INSUFFICIENT_RESOURCES, DLL init ran out of resources";
		case SS_ASPI_IS_SHUTDOWN:		return "SS_ASPI_IS_SHUTDOWN, ASPI DLL is already terminated";
		case SS_BAD_INSTALL:			return "SS_BAD_INSTALL, ASPI is wrongly installed";
		default:						return "unknown return code";
	}	
}

void	CmdDesc(LPSRB srb)
{
	char			disp[255];
	char			str[255];
	int				i;
	SRB_RescanPort	*hdr;										// Same as plain SRB header!
	SRB_ExecSCSICmd	*scsicmd;

	hdr = (LPSRB_RescanPort) srb;
	scsicmd = (LPSRB_ExecSCSICmd) srb;
	
	if (hdr->SRB_Cmd == SC_EXEC_SCSI_CMD)						// SCSI Command
	{
		switch(scsicmd->CDBByte[0])								// DVD command
		{
			case 0xAD:	// Read DVD structure
				AddText("");

				switch(scsicmd->CDBByte[7])						// Which structure to read
				{
					case 0x00:	strcpy(disp, "Read 'Physical' DVD structure "); break;
					case 0x01:	strcpy(disp, "Read 'Copyright' DVD structure"); break;
					case 0x02:	strcpy(disp, "Read 'Disc Key' DVD structure"); break;								
					default:	wsprintf(disp, "Read DVD structure, command %02X", scsicmd->CDBByte[7]); break;
				}

				wsprintf(str, " (layer %d, AGID = %d)", scsicmd->CDBByte[6], (scsicmd->CDBByte[10] >> 6));
				strcat(disp, str);
				break;

			case 0xA4:	// Report KEY command
				AddText("");

				switch(scsicmd->CDBByte[10] & 0x3F)				// Get KEY format
				{
					case 0x00:	strcpy(disp, "Request Authentication Grant ID (AGID)"); break;
					case 0x01:	strcpy(disp, "Request Challenge Key"); break;
					case 0x02:	strcpy(disp, "Request KEY1"); break;
					case 0x04:	strcpy(disp, "Request Title Key"); break;
					case 0x05:	strcpy(disp, "Get status of Authentication Success flag"); break;
					case 0x08:	strcpy(disp, "Report drive region settings"); break;
					case 0x3F:	strcpy(disp, "Invalidate AGID"); break;
					default:	wsprintf(disp, "Report KEY command, command %02X", scsicmd->CDBByte[10] & 0x3F);
								break;
				}

				if ((scsicmd->CDBByte[10] & 0x3F) != 0)
				{
					wsprintf(str, " (AGID = %d)", (scsicmd->CDBByte[10] >> 6));
					strcat(disp, str);
				}

				if ((scsicmd->CDBByte[10] & 0x3F) == 0x04)
				{
					AddText(disp);
					wsprintf(disp, "LBA = %02X%02X%02X%02X", scsicmd->CDBByte[2], scsicmd->CDBByte[3],
																scsicmd->CDBByte[4], scsicmd->CDBByte[5]);
				}
				break;

			case 0xA3:	// Send KEY command
				AddText("");

				switch(scsicmd->CDBByte[10] & 0x3F)				// Get KEY format
				{
					case 0x01:	strcpy(disp, "Sending Challenge Key"); break;
					case 0x03:	strcpy(disp, "Sending KEY2"); break;
					case 0x06:	strcpy(disp, "Set drive region code"); break;
					case 0x3F:	strcpy(disp, "Invalidate AGID"); break;
					default:	wsprintf(disp, "Send KEY command, command %02X", scsicmd->CDBByte[10] & 0x3F);
								break;
				}

				wsprintf(str, " (AGID = %d)", (scsicmd->CDBByte[10] >> 6));
				strcat(disp, str);
				
				switch(scsicmd->CDBByte[10] & 0x3F)
				{
					case 0x01:	AddText(disp);
								wsprintf(disp, "value = %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X", 
										 	 scsicmd->SRB_BufPointer[4],  scsicmd->SRB_BufPointer[5],
											 scsicmd->SRB_BufPointer[6],  scsicmd->SRB_BufPointer[7],
											 scsicmd->SRB_BufPointer[8],  scsicmd->SRB_BufPointer[9],
											scsicmd->SRB_BufPointer[10], scsicmd->SRB_BufPointer[11],
											scsicmd->SRB_BufPointer[12], scsicmd->SRB_BufPointer[13]);

								OutputToFile("challenge.key", scsicmd->SRB_BufPointer, 10, 4);
								break;

					case 0x03:	AddText(disp);
								wsprintf(disp, "value = %02X %02X %02X %02X %02X", 
											scsicmd->SRB_BufPointer[4], scsicmd->SRB_BufPointer[5],
											scsicmd->SRB_BufPointer[6], scsicmd->SRB_BufPointer[7],
											scsicmd->SRB_BufPointer[8]);

								OutputToFile("key2.key", scsicmd->SRB_BufPointer, 5, 4);
								break;

					case 0x06:	AddText(disp);
								wsprintf(disp, "value = %d", scsicmd->SRB_BufPointer[4]);
								break;
				}
				break;

			default:	return;
		}

		// Display latest string build by the previous code block
		AddText(disp);
	
		// Show the full command block
		strcpy(disp, "Full CDB =");
		for (i = 0; i < scsicmd->SRB_CDBLen; i++)
		{
			wsprintf(str, " %02X", scsicmd->CDBByte[i]);
			strcat(disp, str);
		}
		AddText(disp);
	}
}

void	AfterCmd(LPSRB srb)
{
	char			disp[255];
	SRB_RescanPort	*hdr;										// Same as plain SRB header!
	SRB_ExecSCSICmd	*scsicmd;

	hdr = (LPSRB_RescanPort) srb;
	scsicmd = (LPSRB_ExecSCSICmd) srb;
	
	if (hdr->SRB_Cmd == SC_EXEC_SCSI_CMD)						// SCSI Command
	{
		switch(scsicmd->CDBByte[0])								// DVD command
		{
			case 0xAD:	// Read DVD structure
				switch(scsicmd->CDBByte[7])						// Which structure to read
				{
					case 0x00:	AddText("returned:");
								break;

					case 0x01:	if (scsicmd->SRB_BufPointer[4] == 0x01)
								{
									wsprintf(disp, "this DVD disc is copy protected, region mask = %02X", 
												scsicmd->SRB_BufPointer[5]);
								}
								else
									strcpy(disp, "no copyright information on DVD disc");

								AddText(disp);
								break;

					case 0x02:	wsprintf(disp, "Disc key:\t%02X %02X %02X %02X %02X %02X %02X %02X ...", 
												scsicmd->SRB_BufPointer[4], scsicmd->SRB_BufPointer[5],
												scsicmd->SRB_BufPointer[6], scsicmd->SRB_BufPointer[7],
												scsicmd->SRB_BufPointer[8], scsicmd->SRB_BufPointer[9],
												scsicmd->SRB_BufPointer[10], scsicmd->SRB_BufPointer[11]);								
								AddText(disp);

								OutputToFile("disc.key", scsicmd->SRB_BufPointer, 2048, 4);
								break;								
				}
				break;

			case 0xA4:	// Report KEY command
				switch(scsicmd->CDBByte[10] & 0x3F)				// Get KEY format
				{
					case 0x00:	wsprintf(disp, "AGID:\t%d", (scsicmd->SRB_BufPointer[7] >> 6));
								AddText(disp);
								break;

					case 0x01:	wsprintf(disp, "Challenge key:\t%02X %02X %02X %02X %02X %02X %02X %02X %02X %02X",
												scsicmd->SRB_BufPointer[4], scsicmd->SRB_BufPointer[5],
												scsicmd->SRB_BufPointer[6], scsicmd->SRB_BufPointer[7],
												scsicmd->SRB_BufPointer[8], scsicmd->SRB_BufPointer[9],
												scsicmd->SRB_BufPointer[10], scsicmd->SRB_BufPointer[11],
												scsicmd->SRB_BufPointer[12], scsicmd->SRB_BufPointer[13]);
								AddText(disp);

								OutputToFile("challenge.key", scsicmd->SRB_BufPointer, 10, 4);
								break;

					case 0x02:	wsprintf(disp, "KEY1:\t%02X %02X %02X %02X %02X",
												scsicmd->SRB_BufPointer[4], scsicmd->SRB_BufPointer[5],
												scsicmd->SRB_BufPointer[6], scsicmd->SRB_BufPointer[7],
												scsicmd->SRB_BufPointer[8]);
								AddText(disp);

								OutputToFile("key1.key", scsicmd->SRB_BufPointer, 5, 4);
								break;

					case 0x04:	wsprintf(disp, "Title key:\t%02X %02X %02X %02X %02X",
												scsicmd->SRB_BufPointer[5], scsicmd->SRB_BufPointer[6],
												scsicmd->SRB_BufPointer[7], scsicmd->SRB_BufPointer[8],
												scsicmd->SRB_BufPointer[9]);
								AddText(disp);

								if (scsicmd->SRB_BufPointer[4] & 80)
									AddText("this sector contains copyrighted material");

								if (scsicmd->SRB_BufPointer[4] & 40)
								{
									switch(scsicmd->SRB_BufPointer[4] & 30)
									{
										case 0x00: AddText("copying is permitted without restriction"); break;
										case 0x10: AddText("reserved"); break;
										case 0x20: AddText("one generation of copies may be made"); break;
										case 0x30: AddText("copying is not allowed"); break;
									}
								}

								OutputToFile("title.key", scsicmd->SRB_BufPointer, 5, 5);
								break;

					case 0x05:	if (scsicmd->SRB_BufPointer[7] & 1)
									AddText("authentication successfull");
								else
									AddText("authentication was not successfull");
								break;

					case 0x08:	wsprintf(disp, "user changes available: %d. vendor resets available: %d",
												scsicmd->SRB_BufPointer[4] & 7,
												((scsicmd->SRB_BufPointer[4] & 38) >> 3));
								AddText(disp);

								wsprintf(disp, "region mask = %02X", scsicmd->SRB_BufPointer[5]);
								AddText(disp);

								if (scsicmd->SRB_BufPointer[6] == 1)
									AddText("this drive enforces RPC Phase II region protection");
								else
									AddText("this drives does not enforce region protection");
								break;
				}
				break;
		}
	}
}

// 
BOOL	LoadOrgDLL(void)
{
	if (OrgDLL == NULL)
		if (!(OrgDLL = LoadLibraryEx(OrgPath, NULL, 0)))					// Try to open the original WNASPI32.DLL
			return FALSE;													// we failed!
				
	if (!((FARPROC)lpGetSupportInfo = GetProcAddress(OrgDLL, "GetASPI32SupportInfo")))	// Find export "GetASPI32SupportInfo"
	{
		FreeLibrary(OrgDLL);												// failed, exit DLL
		return FALSE;
	}

	if (!((FARPROC)lpSendCommand = GetProcAddress(OrgDLL, "SendASPI32Command")))		// Find export "SendASPI32SupportInfo"
	{
		FreeLibrary(OrgDLL);												// failed, exit DLL
		return FALSE;
	}

	return TRUE;
}

// 
BOOL	InitDLL(void)
{
	DWORD			(*lpGetDLLVer)(void);
	unsigned int	ThreadID;
		
	hThread = CreateThread(NULL, 0, DialogThread, hInst, 0, &ThreadID);	// Create our dialog thread			
	if (hThread == NULL)
		return FALSE;

	while(ListBox == 0);												// Wait until dialog is created
	AddText("Monitor correctly initialized with " OrgDLLName ".");

	if (!((FARPROC)lpGetDLLVer = GetProcAddress(OrgDLL, "GetASPI32DLLVersion")))	// Find export, "GetASPI32DLLVersion"
	{
		AddText("Could not determine original WNASPI32 version, simulating version 1.");
		return TRUE;															// failed, no problem though, not needed
	}

	OrgVer = lpGetDLLVer();														// Get original DLL version and store it
	wsprintf(msg, "Original WNASPI32 version = %d.", OrgVer);
	AddText(msg);

	return TRUE;
}

// Display string in the status listbox
void	AddText(LPSTR lpsz)
{
	SendMessage(ListBox, LB_ADDSTRING, 0, (LPARAM) lpsz);

	fwrite(lpsz, sizeof(char), strlen(lpsz), hOutLog);
	fwrite("\n", sizeof(char), strlen("\n"), hOutLog);
}

// Display timed string in the status listbox
void	AddTimedText(LPSTR lpsz)
{
	char		dum[100];
	SYSTEMTIME	sysTime;

	GetSystemTime(&sysTime);
	wsprintf(dum, "%04d\t%s", sysTime.wMilliseconds, lpsz);

	AddText(dum);
}

void	OutputToFile(char *BaseName, char *Buffer, int Lengte, int Index)
{
	char	FileName[MAX_PATH];
	FILE	*hOut;
	int		i;
	
	Counter++;
	wsprintf(FileName, "%03d_", Counter);
	strcat(FileName, BaseName);

	hOut = fopen(FileName, "wb");
	
	for (i=0; i<Lengte; i++)
		fwrite((Buffer + Index + i), sizeof(char), 1, hOut);

	fclose(hOut);
}
